﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace ContextMenu.ViewModels
{
    class MainViewModel :ViewModelBase
    {
        public MainViewModel()
        {
            Data1 = MyData.GetData(10, 1, 10);
        }

        private MyData currentData;

        public MyData CurrentData
        {
            get { return currentData; }
            set
            {
                currentData = value;
                OnPropertyChanged("CurrentData");
            }
        }
        private ObservableCollection<MyData> data;

        public ObservableCollection<MyData> Data1
        {
            get { return data; }
            set
            {
                data = value;
                OnPropertyChanged("Data1");
            }
        }

        public ICommand AddData
        {
            get
            {
                return new RelayCommand<object>(
                    new Action<object>(
                        (obj) =>
                        {
                            MyData data;
                            data = obj as MyData;
                            EditData edit = new EditData();
                            edit.DataContext = data;
                            edit.ShowDialog();
                        }


                ));
            }
        }

    }

    public class RelayCommand<T> : ICommand
    {

        #region Declarations

        readonly Predicate<T> _canExecute;
        readonly Action<T> _execute;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and 
        /// the command can always be executed.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<T> execute, Predicate<T> canExecute)
        {

            if (execute == null)
                throw new ArgumentNullException("execute");
            _execute = execute;
            _canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested += value;
            }
            remove
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested -= value;
            }
        }



        [DebuggerStepThrough]
        public virtual Boolean CanExecute(Object parameter)
        {
            return _canExecute == null ? true : _canExecute((T)parameter);
        }

        public virtual void Execute(Object parameter)
        {
            _execute((T)parameter);
        }



        #endregion
    }

}
